# -*- coding: utf-8 -*-

__title__ = 'winConnect'
__description__ = 'Communicate Client-Server via Windows NamedPipe.'
__url__ = 'https://github.com/SantaSpeen/winConnect'
__version__ = '0.9.2'
__build__ = 39
__author__ = 'SantaSpeen'
__author_email__ = 'admin@anidev.ru'
__license__ = "MIT"
__copyright__ = 'Copyright 2025 © SantaSpeen'
